<?php

class AVUP_search_video {

	private $params,
			$errors,
			$feed_url,
			$feed_infos,
			$videos,
			$wp_version;

	function __construct( $params ) {

		global $wp_version;
		$this->wp_version = $wp_version;
		$this->params     = $params;

		$custom_post_type = xbox_get_field_value( 'avup-options', 'custom-video-post-type' );

		$video_exists = get_posts(
			array(
				'post_type'      => (string) $custom_post_type != '' ? $custom_post_type : 'post',
				'post_status'    => 'any',
				'posts_per_page' => 1,
				'fields'         => 'ids',
				'meta_query'     => array(
					'relation' => 'AND',
					array(
						'key'     => 'partner',
						'value'   => $params['partner']['id'],
						'compare' => '=',
					),
					array(
						'key'     => 'video_id',
						'value'   => $params['video_id'],
						'compare' => '=',
					),
				),
			)
		);
		// video exist, return video response
		if ( ! empty( $video_exists ) ) {
			$video_id = $video_exists[0];
			$this->set_video_as_already_exists( $video_id );
			return true;
		}

		delete_transient( 'AVUP_FEEDS' );

		$saved_feeds_infos = get_transient( 'AVUP_FEEDS' );
		$feeds_infos       = null;
		$feeds_infos       = $saved_feeds_infos ? $saved_feeds_infos : $this->get_feed();

		// add specific params
		switch ( $this->params['partner']['id'] ) {
			case 'tube8':
				$regex = '/tube8\.com\/(.+)/';
				preg_match( $regex, $this->params['video_url'], $matches, PREG_OFFSET_CAPTURE, 0 );
				if ( isset( $matches[1][0] ) ) {
					$new_params                     = explode( '/', $matches[1][0] );
					$this->params['video_category'] = $new_params[0];
					$this->params['video_slug']     = $new_params[1];
				}
				break;
			case 'youporn':
				$regex['slug'] = '/youporn\.com\/watch\/.+\/(.+)\/?$/';
				preg_match( $regex['slug'], $this->params['video_url'], $matches, PREG_OFFSET_CAPTURE, 0 );
				if ( isset( $matches[1][0] ) ) {
					$this->params['video_slug'] = $matches[1][0];
				}
				break;
			case 'xhamster':
				$regex['slug'] = '/xhamster\.com\/videos\/(.+)-[0-9]+\/?$/';
				preg_match( $regex['slug'], $this->params['video_url'], $matches_url, PREG_OFFSET_CAPTURE, 0 );
				if ( isset( $matches_url[1][0] ) ) {
					$this->params['video_slug'] = $matches_url[1][0];
				}
				$regex['lang']              = '/^http.+\/([a-z]{2})\.xhamster.+$/';
				$this->params['video_lang'] = '';
				preg_match( $regex['lang'], $this->params['video_url'], $matches_lang, PREG_OFFSET_CAPTURE, 0 );
				if ( isset( $matches_lang[1][0] ) ) {
					$this->params['video_lang'] = $matches_lang[1][0] . '.';
				}
				break;
			case 'xvideos':
				$regex['slug'] = '/xvideos\.com\/video[0-9]+\/(.+)\/?$/';
				preg_match( $regex['slug'], $this->params['video_url'], $matches, PREG_OFFSET_CAPTURE, 0 );
				if ( isset( $matches[1][0] ) ) {
					$this->params['video_slug'] = $matches[1][0];
				}
				break;
			default:
				break;
		}

		// success
		if ( false !== $feeds_infos ) {
			$this->feed_infos = $feeds_infos->{$this->params['partner']['id']};
			$this->feed_url   = $this->get_partner_feed_infos( $this->feed_infos->feed_url->data );

			if ( ! $this->feed_url ) {
				WPSCORE()->write_log( 'error', 'Connection to Partner\'s API failed (feed url: <code>' . $this->feed_url . '</code> partner id: <code>' . $this->params['partner']['id'] . '</code>)', __FILE__, __LINE__ );
				return false;
			}
			switch ( $this->params['partner']['data_type'] ) {
				// retrieving videos from native feed
				case 'native':
					return $this->retrieve_videos_from_native_feed();
					break;

				// retrieving videos from json feed
				case 'json':
					return $this->retrieve_videos_from_json_feed();
					break;

				// retrieving videos from xml feed
				default:
					return $this->retrieve_videos_from_xml_feed();
					break;
			}
		} else {
			WPSCORE()->write_log( 'error', 'Connection to API (get_feed) failed (message: <code>' . $response_body->message . '</code>)', __FILE__, __LINE__ );
		}
		return false;
	}

	public function get_video() {
		return $this->video;
	}

	public function get_errors() {
		return $this->errors;
	}

	public function has_errors() {
		return ! empty( $this->errors );
	}





	/********************************/
	/** RETRIVE VIDEO FROM NATIVE */
	/****************************/
	private function retrieve_videos_from_native_feed() {

		$user_agent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/535.6.2 (KHTML, like Gecko) Version/5.2 Safari/535.6.2';

		if ( $this->params['partner']['id'] == 'pornhub' ) {
			$user_agent = 'WordPress/' . $this->wp_version . '; ' . home_url();
		}

		$ch = curl_init();
		curl_setopt( $ch, CURLOPT_HEADER, true );
		curl_setopt( $ch, CURLOPT_REFERER, trim( $this->feed_url ) );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $ch, CURLOPT_URL, trim( $this->feed_url ) );
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $ch, CURLOPT_USERAGENT, $user_agent );

		if ( isset( $this->feed_infos->feed_cookies ) && count( $this->feed_infos->feed_cookies ) > 0 ) {
			$cookies       = $this->feed_infos->feed_cookies;
			$array_cookies = array();
			foreach ( (array) $cookies as $key => $value ) {
				$array_cookies[] = 'Cookie: ' . $key . '=' . $this->get_partner_feed_infos( $value );
			}
			curl_setopt( $ch, CURLOPT_HTTPHEADER, $array_cookies );
		}

		$raw_html = curl_exec( $ch );

		if ( strpos( $raw_html, 'function leastFactor(n)' ) !== false ) {
			$this->errors = array(
				'code'     => 'CURL ERROR ' . $httpcode,
				'message'  => 'Too many requests. Your server IP has been banned from ' . $this->params['partner']['id'],
				'solution' => 'Please use an other partner.',
			);
			WPSCORE()->write_log( 'error', 'Connection to native site (' . $this->params['partner']['id'] . ') failed <code>Too many requests. Your server IP has been banned</code>', __FILE__, __LINE__ );
			curl_close( $ch );
			return false;
		}

		$html = str_get_html( $raw_html );

		$httpcode = curl_getinfo( $ch, CURLINFO_HTTP_CODE );

		curl_close( $ch );

		// region Curl Errors
		if ( 200 != $httpcode ) {
			switch ( $httpcode ) {
				case 403:
				case 0:
					$this->errors = array(
						'code'     => 'CURL ERROR ' . $httpcode,
						'message'  => 'Your Server IP might have been banned for this partner.',
						'solution' => 'Please use an other partner.',
					);
					WPSCORE()->write_log( 'error', 'Connection to native site (' . $this->params['partner']['id'] . ') failed <code>CURL HTTP CODE: ' . $httpcode . '</code>', __FILE__, __LINE__ );
					break;
				case 400:
					$this->errors = array(
						'code'     => 'CURL ERROR ' . $httpcode,
						'message'  => 'Do not panic!',
						'solution' => 'Please contact the WP-Script dev team',
					);
					WPSCORE()->write_log( 'error', 'Connection to native site (' . $this->params['partner']['id'] . ' / ' . $this->feed_url . ') failed <code>CURL HTTP CODE: ' . $httpcode . '</code>', __FILE__, __LINE__ );
					break;
				default:
					$this->errors = array(
						'code'     => 'CURL ERROR ' . $httpcode,
						'message'  => 'Do not panic!',
						'solution' => 'Please contact the WP-Script dev team',
					);
					WPSCORE()->write_log( 'error', 'Connection to native site (' . $this->params['partner']['id'] . ' / ' . $this->feed_url . ') failed <code>CURL HTTP CODE: ' . $httpcode . '</code>', __FILE__, __LINE__ );
					break;
				break;
			}
			return false;
		}
		// endregion
		// TMP récupération des blocks de la page courante
		$video_block = $html->find( $this->get_partner_feed_infos( $this->feed_infos->feed_item_path->node ) );
		$video_block = $video_block[0];

		$video_blocks_counter = count( $video_block );

		// TMP s'il n'y a pas de blocs vidéos, on met $end à true
		if ( $video_blocks_counter == 0 ) {
			$this->errors = array(
				'code'     => 'Video ID not found',
				'message'  => 'Do not panic!',
				'solution' => 'Please try with an other video',
			);
			WPSCORE()->write_log( 'error', 'Video ID not found (' . $this->params['partner']['id'] . ' / ' . $this->feed_url . ')', __FILE__, __LINE__ );
			return false;
		}

		$feed_item = new AVUP_Native_Item( $video_block );
		$feed_item->init( $this->params, $this->feed_infos );
		if ( $feed_item->is_valid() ) {
			$this->video = (array) $feed_item->get_data_for_json();
		} else {
			$this->set_video_as_404();
		}
		return true;
	}





	/******************************/
	/** RETRIVE VIDEO FROM JSON */
	/**************************/
	private function retrieve_videos_from_json_feed() {

		$array_valid_videos     = array();
		$count_valid_feed_items = 0;
		$end                    = false;

		$args = array(
			'timeout'    => 300,
			'sslverify'  => false,
			'user-agent' => 'WordPress/' . $this->wp_version . '; ' . home_url(),
		);

		// add specific params.
		switch ( $this->params['partner']['id'] ) {
			case 'tube8':
				$args['user-agent'] = $_SERVER['HTTP_USER_AGENT'];
				break;
			case 'porncom':
				$args['user-agent'] = $_SERVER['HTTP_USER_AGENT'];
				break;
			case 'pornhub':
				$args['user-agent'] = UAgent::random();
				break;
			default:
				break;
		}

		if ( isset( $this->feed_infos->feed_auth ) ) {
			$args['headers'] = array( 'Authorization' => $this->get_partner_feed_infos( $this->feed_infos->feed_auth->data ) );
		}

		$response = wp_remote_get( $this->feed_url, $args );

		if ( is_wp_error( $response ) ) {
			WPSCORE()->write_log( 'error', 'Retrieving videos from JSON feed failed <code>PARTNER: ' . $this->params['partner']['name'] . '</code> <code>ERROR: ' . json_encode( $response->errors ) . '</code>', __FILE__, __LINE__ );
			return false;
		}

		$response_body = json_decode( wp_remote_retrieve_body( $response ), true );

		if ( isset( $this->feed_infos->feed_item_path->node ) && ! empty( $this->feed_infos->feed_item_path->node ) ) {
			$root       = $this->feed_infos->feed_item_path->node;
			$array_feed = $response_body[ $root ];
		} else {
			$root       = 0;
			$array_feed = $response_body;
		}

		$sub_root      = '';
		$exploded_root = explode( '/', $root );

		// subroot case
		if ( count( $exploded_root ) >= 2 ) {
			$root          = reset( $exploded_root );
			$sub_root      = end( $exploded_root );
			$sub_root_type = explode( ':', $sub_root );

			if ( reset( $sub_root_type ) == 'single' ) {
				$sub_root = end( $sub_root_type );
			}
			$array_feed = $response_body[ $root ];
		}

		if ( $sub_root != '' ) {
			$feed_item = new AVUP_Json_Item( $array_feed[ $sub_root ] );
		} else {
			$feed_item = new AVUP_Json_Item( $array_feed );
		}
		$feed_item->init( $this->params, $this->feed_infos );
		if ( $feed_item->is_valid() ) {
			$this->video = $feed_item->get_data_for_json();
			// video doesn't exist
			if ( $this->video['id'] == '' ) {
				$this->set_video_as_404();
			}
		} else {
			// video doesn't exist
			$this->set_video_as_404();
		}

		return true;
	}

	private function set_video_as_404() {
		$this->video = array(
			'response' => array(
				'code'    => 'danger',
				'message' => 'This video url doesn\'t exist and has returned a 404 error',
			),
		);
	}

	private function set_video_as_already_exists( $video_id ) {
		$this->video = array(
			'response' => array(
				'code'    => 'warning',
				'message' => 'This video already exists in your database (<a href="/wp-admin/post.php?post=' . $video_id . '&action=edit" target="_blank">post ' . $video_id . '</a>)',
			),
		);
	}





	/*****************************/
	/** RETRIVE VIDEO FROM XML */
	/*************************/
	private function retrieve_videos_from_xml_feed() {
		$xml_reader             = new XMLReader();
		$array_valid_videos     = array();
		$count_valid_feed_items = 0;
		$end                    = false;

		if ( ! $xml_reader->open( $this->feed_url ) || ! $xml_reader->read() ) {
			// full xml
			$args     = array(
				'timeout'    => 300,
				'sslverify'  => false,
				'user-agent' => 'WordPress/' . $this->wp_version . '; ' . home_url(),
			);
			$response = wp_remote_get( $this->feed_url, $args );

			if ( ! is_wp_error( $response ) ) {
				$response_body          = wp_remote_retrieve_body( $response );
				$full_xml               = simplexml_load_string( $response_body );
				$count_total_feed_items = count( $full_xml );
				$current_item           = 0;
				while ( $end === false ) {
					$xml_elts  = $full_xml->children()->{$this->feed_infos->feed_item_path->node};
					$feed_item = new AVUP_Xml_Item( $xml_elts[ $current_item ]->asXML() );
					$feed_item->init( $this->params, $this->feed_infos );
					if ( $feed_item->is_valid() ) {
						$array_valid_videos[] = (array) $feed_item->get_data_for_json( $count_valid_feed_items );
						// ++$count_valid_feed_items;
						$end = true;
					}
					if ( $current_item >= ( $count_total_feed_items - 1 ) ) {
						$end = true;
					}
					++$current_item;
				}
				unset( $full_xml, $xml_elts, $feed_item );
			} else {
				if ( isset( $response->errors['http_request_failed'] ) ) {
					wp_die( $response->errors['http_request_failed'] );
				}
			}
		} else {
			// stream xml
			$end = false;
			while ( $xml_reader->read() && $end === false ) {
				switch ( $xml_reader->nodeType ) {
					case ( XMLREADER::ELEMENT ):
						if ( $xml_reader->localName == $this->feed_infos->feed_item_path->node ) {
							$feed_item = new AVUP_Xml_Item( $xml_reader->readOuterXML() );// extends SimpleXMLElement
							$feed_item->init( $this->params, $this->feed_infos );
							if ( $feed_item->is_valid() ) {
								$array_valid_videos = (array) $feed_item->get_data_for_json( $count_valid_feed_items );
								// ++$count_valid_feed_items;
								$end = true;
							}
						}
				}
			}
			$xml_reader->close();
		}
		$this->video = $array_valid_videos;
		return true;
	}


	private function find_native( $video_block, $feed_item ) {

		$array_video_id = array(
			'query'  => $this->get_partner_feed_infos( $this->feed_infos->{$feed_item}->find->query ),
			'offset' => $this->get_partner_feed_infos( $this->feed_infos->{$feed_item}->find->offset ),
			'attr'   => $this->get_partner_feed_infos( $this->feed_infos->{$feed_item}->find->attr ),
		);

		if ( $array_video_id['query'] != '' ) {
			return $video_block->find( (string) $array_video_id['query'], (int) $array_video_id['offset'] )->{$array_video_id['attr']};
		} else {
			return $video_block->{$array_video_id['attr']};
		}
	}

	private function get_feed() {

		// connecting to API
		$api_params = array(
			'license_key'  => WPSCORE()->get_license_key(),
			'signature'    => WPSCORE()->get_client_signature(),
			'server_addr'  => WPSCORE()->get_server_addr(),
			'server_name'  => WPSCORE()->get_server_name(),
			'core_version' => WPSCORE_VERSION,
			'time'         => ceil( time() / 1000 ),
		);

		$args = array(
			'timeout'   => 50,
			'sslverify' => false,
		);

		$base64_params = base64_encode( serialize( $api_params ) );

		$response = wp_remote_get( WPSCORE()->get_api_url( 'avup/get_feed', $base64_params ), $args );

		if ( is_wp_error( $response ) || $response['headers']['content-type'] !== 'application/json; charset=UTF-8' ) {
			if ( isset( $response->errors['http_request_failed'] ) ) {
				WPSCORE()->write_log( 'error', 'Connection to API (get_feed) failed (error: <code>' . json_encode( $response->errors ) . '</code>)', __FILE__, __LINE__ );
				return false;
			}
		}

		$response_body = json_decode( wp_remote_retrieve_body( $response ) );

		if ( $response_body === null ) {
			WPSCORE()->write_log( 'error', 'Connection to API (get_feed) failed (null)', __FILE__, __LINE__ );
			delete_transient( 'AVUP_FEEDS' );
			return false;
		} elseif ( 200 !== $response_body->data->status ) {
			WPSCORE()->write_log( 'error', 'Connection to API (get_feed) failed (status: <code>' . $response_body->data->status . '</code> message: <code>' . $response_body->message . '</code>)', __FILE__, __LINE__ );
			delete_transient( 'AVUP_FEEDS' );
			return false;
		}
		// success
		set_transient( 'AVUP_FEEDS', $response_body->data->feeds_infos, 60 * 5 );
		return $response_body->data->feeds_infos;
	}


	private function get_partner_feed_infos( $partner_feed_item ) {
		preg_match_all( '/<%(.+)%>/U', $partner_feed_item, $results );

		if ( empty( $results ) ) {
			return false;
		}

		foreach ( (array) $results[1] as $result ) {
			$partner_feed_item = str_replace( '<%' . $result . '%>', eval( 'return ' . $result . ';' ), $partner_feed_item );
		}

		return $partner_feed_item;
	}
}
